
# q-expansion of newform 325.6.a.c, downloaded from the LMFDB on 27 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 325
weight = 6
poly_data = [256, -100, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-68, 3, 1], 4]]

hecke_ring_character_values = None
aps_data = [[-2, -1, 0], [-3, 1, 1], [0, 0, 0], [19, 3, 3], [194, -26, 30], [-169, 0, 0], [-277, -77, 93], [-10, 178, -134], [-1232, 72, -152], [-2938, 56, -584], [-820, -148, -268], [6727, 451, -419], [-3952, 858, -250], [-821, 431, -697], [-11409, -33, 807], [4464, -822, -2154], [20830, 1458, -950], [-4888, 1910, 2330], [-18218, -1478, 1666], [26071, -633, 3127], [13438, 2712, 2568], [-19672, -2856, -3576], [-31428, -3808, -4840], [14186, -8864, -432], [-25910, 1396, -500], [-61276, 2994, 1550], [43276, 11108, -6508], [-21740, 8496, 1696], [-92167, 5717, 3131], [-43930, -20664, 9944], [82996, 8100, 17172], [96201, -17843, -8315], [-260368, 2598, -1702], [187673, -707, -4403], [285226, -21732, -188], [-194917, 21059, 25403], [-50798, -34576, -15328], [-41936, -17484, 25932], [71272, -45712, 29208], [133952, 23442, 20990], [-54247, -52427, 5301], [83780, 3618, -49986], [-58790, -42214, -12054], [-261692, 30698, -85594], [-19043, 19917, -22445], [-605626, -16530, 30270], [295313, 66677, -28195], [78289, 14113, 25897], [-864736, -56812, -4148], [55979, -86973, -71115], [-527685, 47891, -24859], [-388659, -57483, 16773], [394262, -23908, 49988], [139648, 56340, -91148], [963055, -93041, -38575], [331540, -6900, -32948], [-580040, -59882, 69050], [661817, -106151, -47711], [583804, 58542, -106878], [-172470, -24352, 55856], [-367628, -98080, 35072], [98363, -87585, 155345], [-738566, 95646, -93498], [-485898, 222750, 103998], [-332639, 28493, -58405], [1459158, -8420, 64420], [-389416, 74796, 267972], [-630839, 128381, 8507], [-808969, 113859, 36851], [-446923, -16111, -26905], [-565496, 124646, 97002], [-2006184, 222096, 100584], [224578, 101170, 50242], [-346172, -276890, 145546], [-365428, 445888, -39416], [-432343, 123433, 52545], [-1130262, -241652, 30292], [-2792270, 152880, -192432], [-1659580, -44258, 312402], [707936, 416330, -121306], [-2772373, 119575, 320871], [2081993, 476805, 4179], [-97267, 298629, -21019], [-775895, -339539, 276091], [1472558, -326106, 203958], [854505, 311677, -366971], [-54746, -7652, -158332], [794218, 55404, -293868], [-1291799, -547547, 32651], [-2030768, 76288, 57832], [1828524, 71472, 402864], [2134169, -234423, 283409], [-1885112, 512624, 27368], [5614829, 175065, 125969], [1855688, 855748, -172148], [-189622, -504622, -434222], [-1481986, 318832, -562128], [7120455, -140373, 139389], [4165468, -552552, 261480], [-1275781, -249389, -59891], [1599133, 965233, -267071], [2942269, 670581, 514555], [-8434069, 33663, 290255], [-7607113, 90211, -319875], [-34753, -921117, 435507], [3725866, -189756, -89172], [-4260728, 702596, 529500], [5491374, -22432, -758848], [2104866, 1606778, -474982], [6582341, 551093, 102251], [356230, 364614, 329046], [8826250, 838528, -2240], [3092312, -919282, -662462], [-8324260, 83920, -1138664], [-5201233, -329913, 46575], [3588430, -552652, -408564], [884854, -758982, -1093278], [-13641710, 204266, -668070], [-5386064, -1042062, 413998], [1911052, -890072, 819224], [-37414, -165236, -713324], [-624725, 174259, -55571], [3270336, -475030, 1101590], [10041704, -561036, 701996], [-1262536, -1288652, 262204], [5185680, 369086, 376034], [15486698, -903796, 9908], [-2142974, 178434, 1326418], [5571646, -413178, 1614486], [5123329, -1488871, -822577], [1494140, 2162712, 810624], [-14422157, 1346763, 783571], [-3139624, 1315032, -560856], [-5725040, -2317382, 63094], [15538138, 1406336, -306624], [-9318640, 407794, -1669394], [-504097, 2459939, 740917], [7649944, -732308, -190172], [2192570, -2526760, 136], [12070319, -1874589, 519749], [-14137156, 689616, -838296], [-9328795, 251273, 1362463], [16224790, -1098466, 574766], [-18938502, 520934, -68338], [-13328602, -131592, 133080], [-7651208, -2553840, 1361032], [11765149, 1123733, 662723], [-711894, 1593052, 710884], [-1643236, -1906768, -239296], [5458647, -398641, 454511], [-17390555, -1139451, 1042563], [-21792883, -1382891, -991749], [-10792703, 3076717, 294757], [10238080, 1368120, 1228264], [-26603273, -1315813, -654829], [-16298262, 1606874, -475318], [22875104, -1404200, 1375768], [-20570972, 2161326, 1200994], [16322050, -1507308, 4138572], [-5456781, -181565, 2757013], [-39291170, 1330194, 553066], [7717241, -743331, 1844531], [-16467107, -1673619, 1770141], [6138751, 349131, -1204877], [16585606, -1489320, -2050568], [6950619, -35077, 344123], [-22118314, 1390934, 2820518], [21536830, 616300, 2291284], [-28367722, -3652532, 59908], [14081130, 1647216, 1399824], [12882570, 381870, 1132422], [28018823, -2626201, -1973191], [-36148616, -1891456, 1327056], [2306824, -2722162, -2476654], [1291754, 596698, 1104058], [8175103, 903987, 2333797], [11873396, -3110216, -3444608], [13975290, 2116860, -3320220], [17378020, -3547748, 2196748], [18122168, 2154214, -2452262], [-863840, 817728, -3486936], [-7710420, 4816440, 1389624], [6068470, -631452, -4622484], [-16696254, 3773604, 5136588], [-22502108, 3678100, -312676], [3987526, 5870880, 1744672], [1527178, 2582536, 365272], [-28158626, 3002178, -3432822], [-13084228, 3973086, 4206258], [-27041744, -2217840, -653624], [37148131, -504701, -1878395], [34355972, -1143136, 2562232], [-21366301, -1631089, -2174161], [30842834, -1203652, 2365252], [-7993951, -5159907, -609451], [-14467466, -1232544, 1675264], [31883096, -4108246, -3181402], [-24046364, -1473458, -2544878], [-29456160, -912866, 3269218], [10192386, -1755222, -1743654], [-6805619, 2468985, -6031217], [-61341112, -789576, -762936], [46140637, 1815949, 1878115], [9981281, -2434551, 2327007], [21472424, 1336764, 5050244], [-58333226, 3753244, -4712668], [36125219, 2072579, 543179], [14261348, 9073304, -2785784], [42392090, -1304992, 6514960], [-15877357, -2311417, -439177], [19424794, 5124852, 3492396], [6144102, 6866256, -2688288], [6700720, -7445832, -2737320], [2550134, 3223442, -399350], [-69868037, -231941, 2938907], [1057124, -4559466, -5227302], [-53285972, 7190412, 2557644], [1636693, -847195, -4175133], [7177780, 4410828, -5022620], [-1042827, -1919859, 2505507], [19992476, 2325930, 5875782], [42484787, 6106763, 2155859], [16285884, -1442762, 468058], [-20150915, -10540787, 3713893], [31217053, 3748905, 6883105], [-64000606, 1900772, 4796060], [-20295455, -5788227, 5960827], [-24738260, 3357164, -7766244], [-55962074, -5680418, -3147986], [9386922, -5774578, 9837638], [-60615590, 6340088, -2038024], [-9734308, -8855784, -6443064], [-43779688, 7990968, 2208288], [-7799253, -12534129, 6890529], [-17938871, -6027827, 344149], [-10250061, -9998661, 2484315], [-83523238, 5743176, -5275416], [-45233207, 8887193, 617583], [16227688, 6196796, -2492868], [-48498790, 6184402, -465246], [-98089466, 3109850, 3671394], [-63945715, 2390297, 7013153], [-79040096, 1423752, -8480424], [-47835205, -3364661, 8340157], [34761866, 361956, -4673764], [-42545580, -2711364, 3073884], [-92592431, 82201, 2184793], [-104483740, -2377776, -2475736], [-19670338, -4796862, -3800310], [70780234, 7173642, -5615174], [-16675937, 4595211, 12676365], [75063146, 11994152, -3667352], [-89286591, -4212759, 4263825], [-3526633, -3417949, -1465843], [35127936, -5766078, 4342398], [49255078, 1978978, -9187078], [-526624, 402534, 333306], [11711674, -659178, -2101122], [39160068, -11795466, 1650810], [-5378894, -5073596, -8716532], [-89694260, 12850172, 11235020], [-126341225, 2122163, -4827813], [-55154929, 14783655, -1430439], [24604606, 11126020, -2451476], [47310248, -13490634, 5229098], [-15780943, 5281589, 865133], [-16778951, 3032501, 6283323], [122935368, 199362, -2040114], [4134364, 1324440, -3971376], [17404676, 7364966, 1897018], [100013732, -4138134, 7203846], [76483786, 21065966, -6155770], [-78177962, 4057024, 1259440], [-31484656, -3104160, -5416080], [-141600110, 4730492, -2924668], [88284424, -9953024, 16928152], [-55473886, 1492958, 11354566], [-81000658, 8144192, -2369728], [-40815580, -4951258, -6240358], [-7682645, -3300129, -6888377], [29890132, -11218140, -16137884], [-23563537, -15646473, 7055487], [33276894, -8934346, 4922630], [113264810, 2815980, 4768404], [-21855632, -8819076, -4071180], [-191771706, 4480582, 352150], [73651135, -11042921, 9772849], [-24890679, 10727841, -14579889], [186046133, -3132987, 9764085], [5473258, 11824952, 2919048], [-43466572, 20004570, 5590070], [-63846997, 10255191, -12631969], [-50125808, 3609558, 13025482], [103510738, -8111178, -7872434], [24306022, -10410932, 18815716], [-17059044, -17845790, 12336334], [165704966, 8399974, -8122250], [-46010468, 19083422, -6545886], [208383851, -10819753, -10052561], [23062, -7939926, 9767538], [57866065, -16217739, -22823973], [90941631, -23077765, -3668995], [-29659870, 22406018, 12169298], [-94419660, 13376632, -18343064], [132102371, -16266369, 3731559], [66311740, 22018562, -3987778], [-276433773, -5148425, 3088591], [-11824732, 888986, -6360346], [8815628, 20963412, -12204052], [-37875308, -29496826, 7985642], [-135436, 3651700, -5907180], [47810351, 7937607, -4786519], [-51026505, -5448917, -12620171], [-63942932, -13379936, -12955952], [-47082235, -3519395, 16405941], [-115754803, -4252227, -8159901], [61527982, 886794, -2281742], [6247464, 221224, 5690896], [-26155664, -7531538, -8647310], [55000142, 11469252, 17817212], [67841780, 19484208, -3989808], [-117236432, 22007158, -20130326], [-165518228, 12067266, 13691134], [204413761, 3861113, -1206871], [195946752, 6303990, -3561654], [-34417846, -11612120, -21049832], [-83273086, 13176894, 20266662], [-61692104, 2570980, -21526148], [117521068, 20255612, -8957940], [-72413648, 19237002, -3828890], [-76964818, -31712496, 17979312], [123750144, 21993426, -8619858], [8021360, -7026784, -15916672], [-94472236, -10838120, -12359088], [21582464, 19170980, -268852], [-107780280, -21788956, -8683924], [26079278, -31225504, 1211456], [-186334148, -8850446, -10159330], [-17087020, -4104906, -752406], [130146802, -8601278, -10305182], [-137169197, 18421871, 15960665], [837677, -27621439, -5674457], [90276309, 4671569, -26381281], [15055070, 4809366, 18360342], [205292201, -4395023, 5865591], [177885273, -3288171, 5217957], [88101686, 25027136, 14505152], [72620908, 27410680, 3149680], [-259676893, -28760677, 5642923], [-188587450, 7945844, 4097356], [-229527106, 1881562, -10598222], [210908458, 11430812, -9621100], [28623965, 28140609, -8773057], [223803472, -13862640, 9002112], [270151190, -22423200, 7596624], [-2820234, -8556944, -31484432], [297520999, 10519247, -4864353], [-70811606, -19593954, 12779350], [-115016163, 14493465, -11012433], [-61396275, -7857523, 16552925], [99720502, 9271988, -5428372], [-224546674, 12405660, 19903844], [-145695626, 44650278, 10279414], [352704464, 4406484, -1561876], [5700820, 36181096, 17445160], [405234151, 5093791, -2948399], [287122049, -8586783, 14429951], [-49557992, -37306784, 32245768], [158711153, -32437631, 6519511], [60125539, 22296159, -23415809], [417804731, 6221839, -12570401], [88541883, -41762821, -14725261], [132168126, 11843928, -15726696], [-349288096, -20255576, 17682424], [60025438, -11549788, -16072084], [46563582, 33146154, 10335234], [-304410338, -7769634, -9570722], [-222459944, 44162134, 14682682], [-119625182, -19349936, -27674704], [35330890, -46812004, 11106404], [-102895986, -54645600, 11719776], [161779154, 10235604, 14147900], [-59421212, -62358276, 1071204], [46188678, -35256072, -5589144], [-341879188, -2071960, 145520], [-36632489, 24757175, -1487097], [349880369, -17855879, -9554231], [-99426692, -31969906, 9071522], [-28393634, 17557722, -48679326], [-27289888, 20455552, 2516976], [123482870, -11716020, 2975796], [256796185, 25355809, -27158641], [43249350, 17541482, 13644002], [-209991425, 1775643, -33221037], [95320429, -10652371, 40373229], [64171591, 12562975, 14865313], [68743775, -50364849, 28951719], [-295618923, -36587331, -7374357], [-87369244, -10643992, 26486072], [-280628095, 21815973, 13293275], [-115734007, 36279381, 33805947], [18771430, -9202440, 6737896], [-184006199, -42911135, 15300481], [-178621068, 500418, 40878510], [29927647, 2693039, 47508201], [52727150, 42926406, -36382890], [191041261, 26332141, -29420597], [-89631536, -40847998, 7980798], [290611102, 24641810, 12753770], [89001176, 19109892, -12600388], [193091191, -28529049, -24700503], [-244310942, 33040072, -31049176], [43034393, 3611277, -4578091], [206221445, -33676639, -27129583], [349503871, 5188079, -3611287], [-296389200, 59309358, 27213042], [240470851, -4523773, -36456693], [-173724224, -4215144, -57794952], [309709054, -41759664, 16484080], [397361942, 3712174, -25876338], [-246035200, -18192242, -27321390], [174159814, -6091420, 38662652], [356621320, -3444680, 43122704], [-585569085, -15623809, 1554119], [180586528, -4155402, -18021558], [-25225570, -19772380, -56084516], [-339756105, -47958261, 31099419], [-17278968, -34279278, 22539438], [-41265304, 8253660, -64694796], [-43059458, -60759882, 7006678]]
